/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module  op_1(
    clk         ,
    rst_n       ,
    key_num     ,
    key_vld     ,
    state_c     ,
    op_1        ,
    result      ,
    op_1_err    
    );

    parameter   KEY_WID     =   4       ;
    parameter   STATE_WID   =   5       ;
    parameter   NUM_WID     =   27      ;
    parameter   OP_1        =   5'b00001;
    parameter   OPER        =   5'b00010;
    parameter   OP_2        =   5'b00100;
    parameter   RESULT      =   5'b01000;
    parameter   ERROR       =   5'b10000;
    
    input                   clk         ;
    input                   rst_n       ;
    input   [KEY_WID-1:0]   key_num ;
    input                   key_vld ;
    input   [STATE_WID-1:0] state_c     ;
    input   [NUM_WID-1:0]   result      ; 

    output  [NUM_WID-1:0]   op_1        ;
    output                  op_1_err    ;
       
    reg     [NUM_WID-1:0]   op_1        ;
    reg                     op_1_err    ;

    wire                    key_num_en  ;
    wire                    key_op_en   ;
    wire                    key_cal_en  ;
    wire                    key_back_en ;
    reg     [STATE_WID-1:0] state_c_ff  ;
    reg                     flag_calc   ;

assign  key_num_en = (key_num==0 || key_num==1 || key_num==2 || key_num==3 || key_num==4 || key_num==5 || key_num==6 || key_num==7 || key_num==8 || key_num==9) && key_vld==1;
assign  key_op_en = (key_num==10 || key_num==11 || key_num==12 || key_num==13) && key_vld==1;
assign  key_cal_en = key_num==15 && key_vld==1; 
assign  key_back_en = key_num==14 && key_vld==1;

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        state_c_ff <= 0;
    end
    else begin
        state_c_ff <= state_c;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        flag_calc <= 0;
    end
    else if(state_c==OP_1)begin
        flag_calc <= 1'b0;
    end
    else if(state_c_ff==OP_2 && state_c==OPER || state_c==RESULT)begin
        flag_calc <= 1'b1;
    end
    else begin
        flag_calc <= flag_calc;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        op_1 <= 0;
    end
    else if(state_c==ERROR)begin
        op_1 <= 10;
    end
    else if((state_c_ff==RESULT || state_c_ff==ERROR) && state_c==OP_1)begin
        op_1 <= key_num;
    end
    else if(state_c==OP_1 && key_back_en==1)begin
        op_1 <= op_1 / 10;
    end
    else if(state_c==OP_1 && key_num_en==1)begin
        op_1 <= (op_1>9999999) ? op_1 : (op_1*10+key_num);
    end
    else if(flag_calc==1)begin
        op_1 <= result;
    end
    else begin
        op_1 <= op_1;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        op_1_err <= 0;
    end
    else if(state_c==OP_1  && key_num_en==1 && op_1>9999999)begin
        op_1_err <= 1'b1;
    end
    else begin
        op_1_err <= 1'b0;
    end
end

endmodule

