/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module  result(
    clk         ,
    rst_n       ,
    key_num     ,
    key_vld     ,
    state_c     ,
    op_1        ,
    oper        ,
    op_2        ,
    result      ,
    result_neg  ,
    result_err  
    );

    parameter   KEY_WID     =   4       ;
    parameter   STATE_WID   =   5       ;
    parameter   NUM_WID     =   27      ;
    parameter   OP_1        =   5'b00001;
    parameter   OPER        =   5'b00010;
    parameter   OP_2        =   5'b00100;
    parameter   RESULT      =   5'b01000;
    parameter   ERROR       =   5'b10000;
    parameter   ADD         =   10      ;
    parameter   DEV         =   11      ;
    parameter   MUL         =   12      ;
    parameter   DIV         =   13      ;

    input                   clk         ;
    input                   rst_n       ;
    input   [KEY_WID-1:0]   key_num ;
    input                   key_vld ;
    input   [STATE_WID-1:0] state_c     ;
    input   [NUM_WID-1:0]   op_1        ;
    input   [KEY_WID-1:0]   oper        ;
    input   [NUM_WID-1:0]   op_2        ;

    output  [NUM_WID-1:0]   result      ;
    output                  result_neg  ;
    output                  result_err  ;
       
    reg     [NUM_WID-1:0]   result      ;
    reg                     result_neg  ;
    reg                     result_err  ;

    wire                    key_op_en   ;
    wire                    key_cal_en  ;
    wire                    calculate   ;
    reg     [STATE_WID-1:0] state_c_ff  ;


assign  key_op_en = (key_num==10 || key_num==11 || key_num==12 || key_num==13) && key_vld==1;
assign  key_cal_en = key_num==15 && key_vld==1;
assign  calculate = (state_c_ff==OP_2 && state_c==OPER || key_cal_en==1);

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        state_c_ff <= 0;
    end
    else begin
        state_c_ff <= state_c;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        result <= 0;
    end
    else if(calculate==1)begin
        case(oper)
            ADD:begin
                if(result_neg==0)
                    result <= op_1 + op_2;
                else 
                    result <= (op_1>op_2) ? (op_1 - op_2) : (op_2 - op_1);
            end
            DEV:begin
                if(result_neg==0)
                    result <= (op_1>op_2) ? (op_1 - op_2) : (op_2 - op_1);
                else 
                    result <= op_1 + op_2;
            end
            MUL:begin
                result <= op_1 * op_2;
            end
            DIV:begin
                result <= op_1 / op_2;
            end
            default:result <= op_1;
        endcase
    end
    else begin
        result <= result;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        result_neg <= 0;
    end
    else if(state_c==OP_1)begin
        result_neg <= 1'b0;
    end
    else if(state_c==ERROR)begin
        result_neg <= 1'b0;
    end
    else if(calculate==1 && oper==DEV && op_1<op_2)begin
        result_neg <= 1'b1;
    end
    else if(calculate==1 && result_neg==1 && oper==ADD && op_1<=op_2)begin
        result_neg <= 1'b0;
    end
    else if(result==0)begin
        result_neg <= 1'b0;
    end
    else begin
        result_neg <= result_neg;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        result_err <= 0;
    end
    else if(state_c==OP_1)begin
        result_err <= 1'b0;
    end
    else if((state_c_ff==OP_2 && state_c==OPER) || (key_cal_en==1 && state_c_ff!=ERROR))begin
        case(oper)
            ADD:begin
                if(result_neg==0)
                    result_err <= (op_1+op_2)>9999_9999 ? 1'b1 : 1'b0;
                else 
                    result_err <= 1'b0;
            end
            DEV:begin
                if(result_neg==1)
                    result_err <= (op_1+op_2)>999_9999 ? 1'b1 : 1'b0;
                else if(op_2>op_1)
                    result_err <= (op_2-op_1)>999_9999 ? 1'b1 : 1'b0;
                else 
                    result_err <= 1'b0;
            end
            MUL:begin
                if(result_neg==1)
                    result_err <= (op_1*op_2)>999_9999 ? 1'b1 : 1'b0;
                else 
                    result_err <= (op_1*op_2)>9999_9999 ? 1'b1 : 1'b0;
            end
            DIV:begin
                if(op_2==0)
                    result_err <= 1'b1;
                else 
                    result_err <= 1'b0;
            end
            default:result_err <= 1'b0;
        endcase 
    end 
    else begin
        result_err <= 1'b0;
    end
end

endmodule
