/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module  work_state(
    clk         ,
    rst_n       ,
    key_num     ,
    key_vld     ,
    result_err  ,
    key_num_out ,
    key_vld_out ,
    state_c
    );

    parameter   KEY_WID     =   4       ;
    parameter   STATE_WID   =   5       ;
    parameter   OP_1        =   5'b00001;
    parameter   OPER        =   5'b00010;
    parameter   OP_2        =   5'b00100;
    parameter   RESULT      =   5'b01000;
    parameter   ERROR       =   5'b10000;

    input                   clk         ;
    input                   rst_n       ;
    input   [KEY_WID-1:0]   key_num     ;
    input                   key_vld     ;
    input                   result_err  ;

    output  [KEY_WID-1:0]   key_num_out ;
    output                  key_vld_out ;
    output  [STATE_WID-1:0] state_c     ;

    reg     [KEY_WID-1:0]   key_num_out ;
    reg                     key_vld_out ;
    reg     [STATE_WID-1:0] state_c     ;

    reg     [KEY_WID-1:0]   key_num_chg ;    
    wire                    key_num_en  ;
    wire                    key_op_en   ;
    wire                    key_cal_en  ;
    wire                    key_back_en ;
    reg     [STATE_WID-1:0] state_n     ;

always  @(*)begin
    case(key_num)
        4'd0   :key_num_chg = 4'd7   ;
        4'd1   :key_num_chg = 4'd8   ;
        4'd2   :key_num_chg = 4'd9   ;
        4'd3   :key_num_chg = 4'd10  ;
        4'd7   :key_num_chg = 4'd11  ;
        4'd8   :key_num_chg = 4'd1   ;
        4'd9   :key_num_chg = 4'd2   ;
        4'd10  :key_num_chg = 4'd3   ;
        4'd11  :key_num_chg = 4'd14  ;
        4'd12  :key_num_chg = 4'd0   ;
        4'd13  :key_num_chg = 4'd12  ;
        4'd14  :key_num_chg = 4'd13  ;
        default:key_num_chg = key_num;
    endcase
end

assign  key_num_en = (key_num_chg==0 || key_num_chg==1 || key_num_chg==2 || key_num_chg==3 || key_num_chg==4 || key_num_chg==5 || key_num_chg==6 || key_num_chg==7 || key_num_chg==8 || key_num_chg==9) && key_vld==1;
assign  key_op_en = (key_num_chg==10 || key_num_chg==11 || key_num_chg==12 || key_num_chg==13) && key_vld==1;
assign  key_cal_en = key_num_chg==15 && key_vld==1; 
assign  key_back_en = key_num_chg==14 && key_vld==1;



always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        state_c <= OP_1 ;
    end
    else begin
        state_c <= state_n;
   end
end

always @(*) begin
    if(result_err)begin
       state_n = ERROR;
   end 
   else begin
       case(state_c)  
        OP_1 :begin
            if(op_12oper_start) 
                state_n = OPER ;
            else if(op_12result_start) 
                state_n = RESULT ;
            else 
                state_n = state_c ;
        end
        OPER :begin
            if(oper2op_2_start) 
                state_n = OP_2 ;
            else if(oper2result_start) 
                state_n = RESULT ;
            else 
                state_n = state_c ;
        end
        OP_2 :begin
            if(op_22oper_start)
                state_n = OPER ;
            else if(op_22result_start) 
                state_n = RESULT ;
            else 
                state_n = state_c ;
        end
        RESULT :begin
            if(result2op_1_start) 
                state_n = OP_1 ;
            else if(result2oper_start) 
                state_n = OPER ;
            else 
                state_n = state_c ;
        end
        ERROR :begin
            if(error2op_1_start) 
                state_n = OP_1 ;
            else 
                state_n = state_c ;
        end
        default : state_n = OP_1 ;
    endcase
end
end

assign op_12oper_start   = state_c==OP_1   && key_op_en ;
assign op_12result_start = state_c==OP_1   && key_cal_en;
assign oper2op_2_start   = state_c==OPER   && key_num_en;
assign oper2result_start = state_c==OPER   && key_cal_en;
assign op_22oper_start   = state_c==OP_2   && key_op_en ;
assign op_22result_start = state_c==OP_2   && key_cal_en;
assign result2op_1_start = state_c==RESULT && key_num_en;
assign result2oper_start = state_c==RESULT && key_op_en ;
assign error2op_1_start  = state_c==ERROR  && key_num_en;

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_num_out <= 0;
    end
    else begin
        key_num_out <= key_num_chg;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_vld_out <= 0;
    end
    else begin
        key_vld_out <= key_vld;
    end
end

endmodule
